/* Virtuino Iot example: ESP32 - Read/Write digital inputs + Sensor reading
 * Supported boards: all ESP32 boards
 * 
 * This example contains the following features
 * 1. How to enable a pin from Virtuino dashboard
 * 2. How to send the state of a pin to Virtuino dashboard
 * 3. How to send sensor values to Virtuino app periodically
 
 * Created by Ilias Lamprou for Virtuino IoT app
 * Modified for DHT22, a capacitive touch soil moisture sensor, a variable (average of soil moisture sensor), and a digital pin to drive a watering pump by Sergio Zárate Pedroche.
 * Updated September 23 2022
 */

#include <WiFi.h>             //Install ESP32 boards ver 1.0.5 or higher. Not working with ver < 1.0.5
#include <WebSocketsServer.h> // Download the library WebSockets by Markus Sattler from arduino library manager (ver 2.3.5)
#include "DHT.h"
uint8_t DHTPIN = 5;  
#define DHTTYPE DHT22   // or DHT22
DHT dht(DHTPIN, DHTTYPE);
#define riego 22
//--- SETTINGS ----------------------------------------
  const char* ssid = "SSID";                      // WIFI network SSID
  const char* password = "Password";              // WIFI network PASSWORD
  WebSocketsServer webSocket = WebSocketsServer(8000); // Default Virtuino Server port = 8000 
//    IPAddress ip(192, 168, 1, 150);                    // 150 is the desired IP Address. The first three numbers must be the same as the router IP
//    IPAddress gateway(192, 168, 1, 1);                 // set gateway to match your network. Replace with your router IP

//---- Enter all the Tags here. You have to use the same Tags on Virtuino variables  
  const char* pin22_tag= "V22";       //tag for pump activation pin
  const char* sensor1_tag= "V0";     // tag for DHT22 relative humidity value
  const char* sensor2_tag= "V1";     // tag for DHT22 temperature value
  const char* sensor3_tag= "V2";     // tag for capacitive touch soil moisture value
  const char* variable1_tag="V21";  //tag for average over 50 of soil moisture sensor
    float h = dht.readHumidity();
    float t = dht.readTemperature();
    int s = (touchRead(T0));
    float p = 0.0;
//---- Enter some variables to hold the last state of the inputs.
  uint8_t sensor1_lastValue=0;
  uint8_t sensor2_lastValue=0;
  uint8_t sensor3_lastValue=0;
  unsigned long lastSensorReadingTime=0;
  uint8_t pin22_lastValue=0; 
  uint8_t variable1_lastValue=0;


//===================================================== sendPinStatus
// It is called every time a new client is connected.
// The void informs Virtuino app about the current pin states and variable values.

void sendPinStatus(){
  
  sendValue(sensor1_tag, String(h));  
  sendValue(sensor2_tag, String(t));
  sendValue(sensor3_tag, String(s));
  sendValue(pin22_tag, String(digitalRead(22)));
  sendValue(variable1_tag, String(p));
  }

//===================================================== onValueReceived
// It is called every time a new value received
  void onValueReceived(String tag, String value){
    Serial.println("Received: tag="+tag+ "  value="+value);
  
    }
//===================================================== setup
//=====================================================
  void setup() {
      Serial.begin(115200);
      while (!Serial) delay(1);    
      connectToWiFiNetwork();
      dht.begin();
      webSocket.begin();
      webSocket.onEvent(webSocketEvent);
  }

//===================================================== loop
//=====================================================
  void loop() {
    pinMode(T0, INPUT);  //pin for capacitive touch soil moisture sensor (T4)
    pinMode(riego, OUTPUT);
  //  float average4 = 0.00;
  for (int i = 0; i < 50; i++) {
    p = p + touchRead(T0);
  }
  p = p/50;
  Serial.println(p);
  if (p >= 10.63)
  {
    digitalWrite(riego, LOW);
  }
  else
  {
    digitalWrite(riego, HIGH);
  }
         webSocket.loop(); 
  if (millis()-lastSensorReadingTime > 5000) {
        float sensor1 = dht.readHumidity();                     // replace with your sensor 1 value
        float sensor2 = dht.readTemperature();                         // replace with your sensor 2 value
        sendValue(sensor1_tag, String(dht.readHumidity()));      // send sensor 1 value
        sendValue(sensor2_tag, String(dht.readTemperature())); // send sensor 2 value
        sendValue(sensor3_tag, String(touchRead(T0)));// send sensor 3 value
        sendValue(pin22_tag, String(digitalRead(22)));
        sendValue(variable1_tag, String(p));
        lastSensorReadingTime=millis();                // store the last sending time of the sensor values
      }
    }

void connectToWiFiNetwork(){
  Serial.println("Connecting to "+String(ssid));
  // If you don't want to config IP manually disable the next two lines and the disables IPAddress variables  at the top of the code
  //IPAddress subnet(255, 255, 255, 0);        // set subnet mask to match your network
//  WiFi.config(ip, gateway, subnet);          // If you don't want to config IP manually disable this line
  WiFi.mode(WIFI_STA);                         // Configure the module as station only.
  WiFi.begin(ssid, password);
   while (WiFi.status() != WL_CONNECTED) {
     delay(500);
     Serial.print(".");
    }
   Serial.println("\nWiFi connected");
   Serial.println(WiFi.localIP());              // Insert this IP on Virtuino IoT server settings
}

/*=====================================================
  ======================  UTILS =======================
  =====================================================
  You don't need to make changes to the code below
*/
//===================================================== connectToWiFiNetwork

 
//===================================================== sendValue
// This function sends a value to a Tag. 
// The tag and the value are converted to a json message before sending 

bool sendValue(const char* tag, String value){
  String json = "{\"";
  json+=tag;
  json+="\":\"";
  json+=value;
  json+="\"}";
  Serial.println("Send: "+json);
  return webSocket.broadcastTXT(json);     // This function sends the message to all connected clients.
}

//===================================================== webSocketEvent
//This is the server handler. It receives all the messages

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {
    switch(type) {
        case WStype_DISCONNECTED:
            Serial.printf("[%u] Disconnected!\n", num);
            break;
        case WStype_CONNECTED:{ 
            IPAddress ip = webSocket.remoteIP(num);
            Serial.printf("[%u] New client connected - IP: %d.%d.%d.%d \n", num, ip[0], ip[1], ip[2], ip[3]);
            sendPinStatus();         // send the initial pin and variable values to the connected clients
            break;
        }
        case WStype_TEXT:  // a new message received 
            Serial.printf("[%u] Received: %s\n", num, payload);
            //-- The incoming payload is a json message. The following code extracts the value from json without extra library
            String str = (char*)payload;
            int p1 = str.indexOf("{\"");
            if (p1==0) {
              int p2 = str.indexOf("\":");
              if (p2>p1) {
                String tag = str.substring(p1+2,p2);
                p1 = str.indexOf(":\"",p2);
                if (p1>0) {
                   p2 = str.lastIndexOf("\"}");  
                   if (p2>0){
                      String value = str.substring(p1+2,p2);
                      onValueReceived(tag,value);        
                    }
                  }
                }
            }
            break;
    }
}

 //============================================================== vDelay
  void vDelay(int delayInMillis){long t=millis()+delayInMillis;while (millis()<t) webSocket.loop();}
  
  
